"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const _ = require("lodash");
const tyr_1 = require("../tyr");
function extractProjection(opts) {
    return opts.projection || opts.fields || opts.project;
}
exports.extractProjection = extractProjection;
function resolveProjection(projections, projection) {
    function inner(projection) {
        if (_.isArray(projection)) {
            const net = {};
            for (const proj of projection) {
                _.merge(net, inner(proj));
            }
            return net;
        }
        else if (_.isObject(projection)) {
            return projection;
        }
        else if (_.isString(projection)) {
            const rslt = projections[projection];
            if (!rslt) {
                throw new Error(`No project named "${projection}"`);
            }
            return rslt;
        }
        else {
            throw new Error('invalid projection');
        }
    }
    const rslt = inner(projection), minimal = projections && projections.$minimal;
    if (minimal) {
        const net = {};
        const m = rslt.$minimal;
        if (m !== undefined) {
            if (m)
                _.merge(net, minimal);
            _.merge(net, rslt);
            delete net.$minimal;
        }
        else {
            _.merge(net, minimal);
            _.merge(net, rslt);
        }
        return net;
    }
    else if (rslt.$minimal !== undefined) {
        return _.omit(rslt, ['$minimal']);
    }
    else {
        return rslt;
    }
}
exports.resolveProjection = resolveProjection;
/**
 * Mongo projections can take paths like 'foo.bar'.  This returns projections that have
 * been flattened to the top-level, for example:
 *
 * flattenProjection({
 *   'name.first': 1,
 *   'name.last': 1,
 *   'email': 1
 * })
 *
 * returns
 *
 * {
 *   name: 1,
 *   email: 1
 * }
 */
function flattenProjection(opts) {
    const proj = extractProjection(opts);
    if (!proj)
        return proj;
    const newProj = {};
    for (const pathName in proj) {
        const idx = pathName.indexOf('.');
        newProj[idx >= 0 ? pathName.substring(0, idx) : pathName] = 1;
    }
    delete opts.fields;
    delete opts.project;
    opts.projection = newProj;
}
exports.flattenProjection = flattenProjection;
/** @isomorphic */
tyr_1.default.projectify = function (value) {
    const projection = {};
    if (Array.isArray(value)) {
        for (const path of value) {
            if (path)
                path.projectify(projection);
        }
    }
    else {
        for (const name in value)
            projection[name] = 1;
    }
    return projection;
};
//# sourceMappingURL=projection.js.map